@@ -3,7 +3,7 @@
  This file is part of SLRN.
 
  Copyright (c) 1994, 1999 John E. Davis <davis@space.mit.edu>
- Copyright (c) 2001, 2002 Thomas Schultz <tststs@gmx.de>
+ Copyright (c) 2001-2003 Thomas Schultz <tststs@gmx.de>
 
  This program is free software; you can redistribute it and/or modify it
  under the terms of the GNU General Public License as published by the Free
@@ -69,7 +69,7 @@
 /*{{{ Global Variables */
 
 int Slrn_Query_Group_Cutoff = 100;
-int Slrn_Groups_Dirty;	       /* greater than 0 if need to write newsrc */
+int Slrn_Groups_Dirty;	       /* 1 == need to write newsrc */
 int Slrn_List_Active_File = 0;
 int Slrn_Write_Newsrc_Flags = 0;       /* if 1, do not save unsubscribed 
 					* if 2, do not save new unsubscribed.
@@ -129,119 +129,75 @@ static void remove_group_entry (Slrn_Group_Type *);
 
 /*{{{ Functions that deal with Group Range */
 
-
-/* Note: This routine is NOT very robust.  It assumes that this function
- * is called in an ordered way such that the implied range is always increasing.
- * This is why the range is re-built in art.c:update_ranges.  Yes, it is ugly.
- * See also slrn_group_mark_article_as_read for something more random.
- */
-void slrn_add_group_ranges (Slrn_Group_Type *g, int min, int max) /*{{{*/
+static int count_unread (Slrn_Range_Type *r)
 {
-   Slrn_Range_Type *r, *next;
-   int unread;
-   
-   if ((max < min) || (g == NULL)) return;
-   
-   /* The first one is range of articles on server so expand max to cover
-    * the range of articles nolonger available.
-    */
-   next = &g->range;
-   if (max < next->min) 
+   int unread = r->max;
+
+   while (r->next != NULL)
      {
-	/* If we have already expanded the range up to range currently available
-	 * at server and we are now trying to add another range below available
-	 * range, do not bother.
-	 */
-	if (next->next != NULL) return;
-	max = next->min - 1;
+	r = r->next;
+	unread -= r->max - r->min + 1;
      }
    
-   /* Count number unread */
-   unread = next->max;
-   while (next->next != NULL)
-     {
-	next = next->next;
-	unread -= next->max - next->min + 1;
-     }
-   
-   /* check to see if a merge is possible */
-   if ((min <= next->max + 1)
-       && (next != &g->range))
-     {
-	next->max = max;
-     }
-   else
-     {
-	r = (Slrn_Range_Type *) slrn_safe_malloc (sizeof(Slrn_Range_Type));
-	
-	r->next = next->next;
-	next->next = r;
-	r->prev = next;
-	
-	r->min = min;
-	r->max = max;
-	
-#if 0
-	/* For this case, min should be 1 */
-	if (next == &g->range)
-	  {
-	     min = r->min = 1;
-	  }
-#endif
-     }
-   
-   unread -= max - min + 1;
-   
-   if (unread < 0) unread = 0;
-   g->unread = unread;
-   Slrn_Groups_Dirty = 1;
+   return unread;
 }
 
+void slrn_group_recount_unread (Slrn_Group_Type *g)
+{
+   /* Make sure old (unavailable) messages are marked read */
+   if (g->range.min>1)
+     g->range.next = slrn_ranges_add (g->range.next, 1, g->range.min-1);
+   g->unread = count_unread (&g->range);
+   if (g->unread < 0)
+     g->unread = 0;
+}   
+
+static int is_article_requested (Slrn_Group_Type *g, long num) /*{{{*/
+{
+#if SLRN_HAS_SPOOL_SUPPORT
+   if (Slrn_Server_Id != SLRN_SERVER_ID_SPOOL) return 0;
+   
+   if (g->requests_loaded == 0)
+     {
+	g->requests = slrn_spool_get_requested_ranges (g->group_name);
+	g->requests_loaded = 1;
+     }
+
+   return slrn_ranges_is_member (g->requests, num);
+#else
+   (void) g; (void) num;
+   return 0;
+#endif
+}
 /*}}}*/
 
 static void group_mark_article_as_read (Slrn_Group_Type *g, long num) /*{{{*/
 {
-   Slrn_Range_Type *r, *r1, *newr;
+   Slrn_Range_Type *r;
    
-   r1 = &g->range;
-   if (r1->max < num)  /* not at server yet so update our data */
+   /* Never mark articles as read if their body has been requested. */
+   if (is_article_requested (g, num)) return;
+   
+   r = &g->range;
+   if (r->max < num)  /* not at server yet so update our data */
      {
-	r1->max = num;
+	r->max = num;
 	g->unread += 1;
      }
    
-   r = r1->next;
+   r = r->next;
    
    while (r != NULL)
      {
 	/* Already read */
 	if ((num <= r->max) && (num >= r->min)) return;
 	if (num < r->min) break;
-	r1 = r;
 	r = r->next;
      }
    
    if (g->unread > 0) g->unread -= 1;
    Slrn_Groups_Dirty = 1;
-   if ((r != NULL) && (r->min == num + 1))
-     {
-	r->min = num;
-	return;
-     }
-   
-   if ((r1->max + 1 == num) && (r1 != &g->range))
-     {
-	r1->max = num;
-	return;
-     }
-   
-   newr = (Slrn_Range_Type *) slrn_safe_malloc (sizeof (Slrn_Range_Type));
-   
-   newr->min = newr->max = num;
-   newr->next = r;
-   if (r != NULL) r->prev = newr;
-   newr->prev = r1;
-   r1->next = newr;
+   g->range.next = slrn_ranges_add (g->range.next, num, num);   
 }
 
 /*}}}*/
@@ -386,6 +342,7 @@ static int group_sync_group_with_server (Slrn_Group_Type *g, int *minp, int *max
      {
 	slrn_error (_("Group %s is bogus%s."), group,
 		    Slrn_Drop_Bogus_Groups ? _(" - dropping it") : "");
+	Slrn_Saw_Warning = 1;
 	if (Slrn_Drop_Bogus_Groups)
 	  remove_group_entry (g);
 	return -1;
@@ -403,30 +360,13 @@ static int group_sync_group_with_server (Slrn_Group_Type *g, int *minp, int *max
 
 /*}}}*/
 
-static void free_group_ranges (Slrn_Group_Type *g) /*{{{*/
-{
-   Slrn_Range_Type *r, *rnext;
-   
-   if (g == NULL) return;
-   
-   r = g->range.next;
-   while (r != NULL)
-     {
-	rnext = r->next;
-	SLFREE (r);
-	r = rnext;
-     }
-   g->range.next = NULL;
-}
-
-/*}}}*/
-
 static void free_newsgroup_type (Slrn_Group_Type *g)
 {
    if (g == NULL)
      return;
    slrn_free (g->descript);
-   free_group_ranges (g);
+   slrn_ranges_free (g->range.next);
+   slrn_ranges_free (g->requests);
    slrn_free (g->group_name);
    slrn_free ((char *) g);
 }
@@ -441,9 +381,15 @@ static void free_unsubscribed_group_type (Unsubscribed_Slrn_Group_Type *ug)
 
 void slrn_catchup_group (void) /*{{{*/
 {
-   if (Slrn_Group_Current_Group == NULL) return;
-   free_group_ranges (Slrn_Group_Current_Group);
-   slrn_add_group_ranges (Slrn_Group_Current_Group, 1, Slrn_Group_Current_Group->range.max);
+   if ((Slrn_Group_Current_Group == NULL)||
+       (Slrn_Group_Current_Group->unread==0))
+     return;
+   
+   Slrn_Group_Current_Group->range.next =
+     slrn_ranges_add(Slrn_Group_Current_Group->range.next, 1,
+		     Slrn_Group_Current_Group->range.max);
+   Slrn_Group_Current_Group->unread = 0;
+   Slrn_Groups_Dirty = 1;
    Slrn_Group_Current_Group->flags |= GROUP_TOUCHED;
 }
 
@@ -453,8 +399,10 @@ void slrn_uncatchup_group (void) /*{{{*/
 {
    if (Slrn_Group_Current_Group == NULL)
      return;
-   free_group_ranges (Slrn_Group_Current_Group);
-   slrn_add_group_ranges (Slrn_Group_Current_Group, 1, 1);
+   slrn_ranges_free (Slrn_Group_Current_Group->range.next);
+   Slrn_Group_Current_Group->range.next = NULL;
+   slrn_group_recount_unread (Slrn_Group_Current_Group);
+   Slrn_Groups_Dirty = 1;
    Slrn_Group_Current_Group->flags |= GROUP_TOUCHED;
 }
 
@@ -511,6 +459,9 @@ static Slrn_Group_Type *create_group_entry (char *name, unsigned int len, /*{{{*
      }
    
    g = (Slrn_Group_Type *) slrn_safe_malloc (sizeof (Slrn_Group_Type));
+   g->requests = NULL;
+   g->requests_loaded = 0;
+   
    g->group_name = slrn_safe_malloc (len + 1);
    strncpy (g->group_name, name, len);
    g->group_name [len] = 0;
@@ -605,7 +556,6 @@ static int add_group (char *name, unsigned int len, /*{{{*/
 		      unsigned int subscribe_flag, int query_server,
 		      int create_flag)
 {
-   char ch;
    Slrn_Group_Type *g;
    
    g = find_group_entry (name, len);
@@ -652,24 +602,8 @@ static int add_group (char *name, unsigned int len, /*{{{*/
    /* find ranges for this */
    name += len;			       /* skip past name */
    if (*name) name++;			       /* skip colon */
-   while (1)
-     {
-	int min, max;
-	/* skip white space and delimiters */
-	while (((ch = *name) != 0) && ((ch <= ' ') || (ch == ','))) name++;
-	if ((ch < '0') || (ch > '9')) break;
-	min = atoi (name++);
-	while (((ch = *name) != 0) && (ch >= '0') && (ch <= '9')) name++;
-	if (ch == '-')
-	  {
-	     name++;
-	     max = atoi (name);
-	     while (((ch = *name) != 0) && (ch >= '0') && (ch <= '9')) name++;
-	  }
-	else max = min;
-	
-	slrn_add_group_ranges (Slrn_Group_Current_Group, min, max);
-     }
+   g->range.next = slrn_ranges_from_newsrc_line (name);
+   slrn_group_recount_unread (g);
 
    if ((g->range.next != NULL)
        && (g->range.next->min < g->unread)
@@ -788,10 +722,10 @@ static int find_group (char *name) /*{{{*/
 
 /*}}}*/
 
-/* origpat needs enough space for 256 chars */
+/* origpat needs enough space for SLRL_DISPLAY_BUFFER_SIZE chars */
 static SLRegexp_Type *read_group_regexp (char *prompt, char *origpat) /*{{{*/
 {
-   static char pattern[256];
+   static char pattern[SLRL_DISPLAY_BUFFER_SIZE];
    
    if (slrn_read_input (prompt, NULL, pattern, 1, 0) <= 0) return NULL;
    
@@ -943,18 +877,29 @@ int slrn_group_select_group (void) /*{{{*/
 
    if ((prefix & 1) ||
        ((n > Slrn_Query_Group_Cutoff)
-	&& (Slrn_Query_Group_Cutoff > 0)))
+	&& (Slrn_Query_Group_Cutoff > 0)) ||
+       ((n > -Slrn_Query_Group_Cutoff)
+	&& (Slrn_Query_Group_Cutoff < 0)))
      {
 	char int_prompt_buf[256];
-	slrn_snprintf (int_prompt_buf, sizeof (int_prompt_buf),
-		       _("%s: Read how many? "),
-		       Slrn_Group_Current_Group->group_name);
-	if ((-1 == slrn_read_integer (int_prompt_buf, &n, &n))
-	    || (n <= 0))
+	if ((prefix & 1) || (Slrn_Query_Group_Cutoff > 0))
 	  {
-	     slrn_clear_message ();
-	     Slrn_Full_Screen_Update = 1;
-	     return 0;
+	     slrn_snprintf (int_prompt_buf, sizeof (int_prompt_buf),
+			    _("%s: Read how many? "),
+			    Slrn_Group_Current_Group->group_name);
+	     if ((-1 == slrn_read_integer (int_prompt_buf, &n, &n))
+		 || (n <= 0))
+	       {
+		  slrn_clear_message ();
+		  Slrn_Full_Screen_Update = 1;
+		  return 0;
+	       }
+	  }
+	else
+	  {
+	     slrn_message_now (_("Only downloading %d of %d articles."),
+			       -Slrn_Query_Group_Cutoff, n);
+	     n = -Slrn_Query_Group_Cutoff;
 	  }
 	
 	if ((0 == prefix)
@@ -1097,6 +1042,10 @@ static void set_current_group (void) /*{{{*/
      }
    Slrn_Group_Current_Group = g;
    
+   /* When there are less than SCREEN_HEIGHT-2 groups, they should all get
+    * displayed; scroll to the top of the buffer to ensure this. */
+   while (SLscroll_prev_n (&Group_Window, 1000));
+   (void) SLscroll_find_top (&Group_Window);
    find_line_num ();
    
    Slrn_Full_Screen_Update = 1;
@@ -1179,7 +1128,7 @@ static void refresh_groups_cmd (void) /*{{{*/
    if (Slrn_List_Active_File)
      {
 	read_and_parse_active (0);
-	if (c != NULL)
+	if ((Slrn_Server_Obj->sv_id == SERVER_ID_INN) && (c != NULL))
 	  /* hack: avoid a problem with inn not updating the high water mark */
 	  {
 	     Slrn_Group_Type *a = c, *b;
@@ -1218,7 +1167,7 @@ static void refresh_groups_cmd (void) /*{{{*/
 
 static void generic_group_search (int dir) /*{{{*/
 {
-   static char search_str[256];
+   static char search_str[SLRL_DISPLAY_BUFFER_SIZE];
    char* prompt;
    Slrn_Group_Type *g;
    unsigned int n;
@@ -1284,7 +1233,7 @@ int slrn_add_group (char *group) /*{{{*/
 
 static void add_group_cmd (void) /*{{{*/
 {
-   char group[256];
+   char group[SLRL_DISPLAY_BUFFER_SIZE];
    
    *group = 0;
    if (slrn_read_input (_("Add group: "), NULL, group, 1, 0) > 0)
@@ -1577,7 +1526,7 @@ static void toggle_list_all_groups1 (int hide_flag) /*{{{*/
    else
      {
 	SLRegexp_Type *re;
-	char origpat[256];
+	char origpat[SLRL_DISPLAY_BUFFER_SIZE];
 	
 	if (NULL == (re = read_group_regexp (_("List Groups (e.g., comp*unix*): "),
 					     origpat)))
@@ -1734,9 +1683,9 @@ static void select_group_cmd (void)
 void slrn_post_cmd (void) /*{{{*/
 {
    char *name;
-   char group[256];
-   char followupto[256];
-   char subj[256];
+   char group[SLRL_DISPLAY_BUFFER_SIZE];
+   char followupto[SLRL_DISPLAY_BUFFER_SIZE];
+   char subj[SLRL_DISPLAY_BUFFER_SIZE];
    
    if (Slrn_Post_Obj->po_can_post == 0)
      {
@@ -2526,7 +2475,7 @@ static void read_and_parse_active (int create_flag) /*{{{*/
 	status = Slrn_Server_Obj->sv_read_line (line, sizeof(line));
 	if (status == -1)
 	  {
-	     Slrn_Groups_Dirty = 0;
+	     /*	     Slrn_Groups_Dirty = 0;*/ /* Huh? */
 	     slrn_exit_error (_("Read from server failed"));
 	  }
 	if (status == 0)
@@ -2665,7 +2614,7 @@ static int read_and_parse_newsrc_file (void)
 	/* perform a re-arrangement to match arrangement in the
 	 * newsrc file
 	 */
-	if (Slrn_List_Active_File && (ch !=  '!'))
+	/*if (Slrn_List_Active_File && (ch !=  '!'))*/
 	  last_group = place_group_in_newsrc_order (last_group);
      }
    vclose (vp);
@@ -2745,7 +2694,7 @@ int slrn_write_newsrc (int auto_save) /*{{{*/
 {
    Slrn_Group_Type *g;
    Slrn_Range_Type *r;
-   char backup_file[SLRN_MAX_PATH_LEN], autosave_file[SLRN_MAX_PATH_LEN];
+   char autosave_file[SLRN_MAX_PATH_LEN];
    char *newsrc_filename;
    static FILE *fp;
    int max;
@@ -2769,18 +2718,13 @@ int slrn_write_newsrc (int auto_save) /*{{{*/
 #ifdef VMS
    slrn_snprintf (autosave_file, sizeof (autosave_file), "%s-as",
 		  Slrn_Newsrc_File);
-   slrn_snprintf (backup_file, sizeof (backup_file), "%s-bak",
-		  Slrn_Newsrc_File);
 #else
 # ifdef SLRN_USE_OS2_FAT
    slrn_os2_make_fat (autosave_file, sizeof (autosave_file), Slrn_Newsrc_File,
 		      ".as");
-   slrn_os2_make_fat (backup_file, sizeof (backup_file), Slrn_Newsrc_File,
-		      ".bak");
 # else
    slrn_snprintf (autosave_file, sizeof (autosave_file), "%s.as",
 		  Slrn_Newsrc_File);
-   slrn_snprintf (backup_file, sizeof (backup_file), "%s~", Slrn_Newsrc_File);
 # endif
 #endif
    
@@ -2811,17 +2755,13 @@ int slrn_write_newsrc (int auto_save) /*{{{*/
 	/* Create a temp backup file.  Delete it later if user 
 	 * does not want backups.
 	 */
-	have_backup = 1;
-	if (-1 == rename (newsrc_filename, backup_file))
-	  {
-	     have_backup = 0;
-	  }
+	have_backup = (0 == slrn_create_backup (newsrc_filename));
      }
 
    if (NULL == (fp = fopen (newsrc_filename, "w")))
      {
 	slrn_error (_("Unable to open file %s for writing."), newsrc_filename);
-	if (have_backup) (void) rename (backup_file, newsrc_filename);
+	if (have_backup) slrn_restore_backup (newsrc_filename);
 	slrn_init_hangup_signals (1);
 	return -1;
      }
@@ -2862,34 +2802,19 @@ int slrn_write_newsrc (int auto_save) /*{{{*/
 	max = g->range.max;
 	if (r != NULL)
 	  {
+	     int max_newsrc_number=0;
+	     /* Make this check because the unsubscribed group
+	      * range may not have been initialized from the server.
+	      */
+	     if ((max != -1) && (g->range.min != -1)
+		 && ((g->flags & GROUP_UNSUBSCRIBED) == 0))
+	       max_newsrc_number = max;
+	     
 	     if (EOF == putc (' ', fp))
 	       goto write_error;
-	     
-	     while (1)
-	       {
-		  /* Make this check because the unsubscribed group
-		   * range may not have been initialized from the server.
-		   */
-		  if ((max != -1) && (g->range.min != -1)
-		      && ((g->flags & GROUP_UNSUBSCRIBED) == 0))
-		    {
-		       if (r->min > max) break;
-		       if (r->max > max) r->max = max;
-		    }
-		  
-		  if (r->min != r->max)
-		    {
-		       if (fprintf (fp, "%d-%d", r->min, r->max) < 0)
-			 goto write_error;
-		    }
-		  else if (fprintf (fp, "%d", r->min) < 0)
-		    goto write_error;
-		  
-		  r = r->next;
-		  if (r == NULL) break;
-		  if (EOF == putc (',', fp))
-		    goto write_error;
-	       }
+
+	     if (-1 == slrn_ranges_to_newsrc_file (r, max_newsrc_number, fp))
+	       goto write_error;
 	  }
 	else if (g->range.min == 2)
 	  {
@@ -2914,7 +2839,7 @@ int slrn_write_newsrc (int auto_save) /*{{{*/
 
    if (Slrn_No_Backups)
      {
-	if (have_backup) slrn_delete_file (backup_file);
+	if (have_backup) slrn_delete_backup (newsrc_filename);
      }
    
    if (newsrc_filename == Slrn_Newsrc_File)
@@ -2935,7 +2860,7 @@ int slrn_write_newsrc (int auto_save) /*{{{*/
    slrn_error (_("Write to %s failed! Disk Full?"), newsrc_filename);
    
    /* Put back orginal file */
-   if (have_backup) (void) rename (backup_file, newsrc_filename);
+   if (have_backup) slrn_restore_backup (newsrc_filename);
    
    slrn_init_hangup_signals (1);
    return -1;
