@@ -14,8 +14,8 @@
  more details.
 
  You should have received a copy of the GNU General Public License along
- with this program; if not, write to the Free Software Foundation, Inc., 675
- Mass Ave, Cambridge, MA 02139, USA. 
+ with this program; if not, write to the Free Software Foundation, Inc.,
+ 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 extern unsigned long slrn_compute_hash (unsigned char *, unsigned char *);
 
