@@ -3,7 +3,7 @@
  This file is part of SLRN.
 
  Copyright (c) 1994, 1999 John E. Davis <davis@space.mit.edu>
- Copyright (c) 2001, 2002 Thomas Schultz <tststs@gmx.de>
+ Copyright (c) 2001-2003 Thomas Schultz <tststs@gmx.de>
 
  This program is free software; you can redistribute it and/or modify it
  under the terms of the GNU General Public License as published by the Free
@@ -281,12 +281,18 @@ static int parsed_headers_to_xover (int id, Slrn_XOver_Type *xov) /*{{{*/
    if (NULL == (bytes = Xover_Headers[BYTES_OFFSET].value))
      bytes = "";
    
-   xov->subject_malloced = slrn_strmalloc (subj, 0);
-   if (xov->subject_malloced == NULL)
-     return -1;
+   len = strlen (subj) + strlen (from) + 2;
+   buf = slrn_malloc (len, 0, 1);
+   if (buf == NULL) return -1;
    
-   len = strlen (from) + strlen (date) + strlen (msgid) + strlen (refs) +
-     strlen (xref) + 5;
+   xov->subject_malloced = buf;
+   strcpy (buf, subj); /* safe */
+   buf += strlen (subj) + 1;
+   
+   xov->from = buf;
+   strcpy (buf, from); /* safe */
+   
+   len = strlen (date) + strlen (msgid) + strlen (refs) + strlen (xref) + 4;
    
    buf = slrn_malloc (len, 0, 1);
    if (buf == NULL)
@@ -295,11 +301,7 @@ static int parsed_headers_to_xover (int id, Slrn_XOver_Type *xov) /*{{{*/
 	return -1;
      }
    
-   xov->from_malloced = buf;
-   strcpy (buf, from); /* safe */
-   buf += strlen (from) + 1;
-
-   xov->date = buf;
+   xov->date_malloced = buf;
    strcpy (buf, date); /* safe */
    buf += strlen (date) + 1;
    
@@ -313,7 +315,6 @@ static int parsed_headers_to_xover (int id, Slrn_XOver_Type *xov) /*{{{*/
    
    xov->xref = buf;
    strcpy (buf, xref); /* safe */
-   buf += strlen (xref) + 1;
 
    xov->bytes = atoi (bytes);
    xov->lines = atoi (lines);
@@ -321,12 +322,19 @@ static int parsed_headers_to_xover (int id, Slrn_XOver_Type *xov) /*{{{*/
 #ifndef SLRNPULL_CODE
    if (id == -1)
      id = extract_id_from_xref (xov->xref);
+# if SLRN_HAS_SPOOL_SUPPORT
+   if (Slrn_Spool_Check_Up_On_Nov && (xov->bytes == 0) && (id != -1))
+     {
+	int size = Slrn_Server_Obj->sv_get_article_size (id);
+	if (size != -1) xov->bytes = size;
+     }
+# endif
 #endif
    
    xov->add_hdrs = copy_add_headers (Additional_Headers, 0);
 
    xov->id = id;
-
+   
    return 0;
 }
 
@@ -374,8 +382,8 @@ void slrn_map_xover_to_header (Slrn_XOver_Type *xov, Slrn_Header_Type *h)
    
    h->subject = xov->subject_malloced;
    h->number = xov->id;
-   h->from = xov->from_malloced;
-   h->date = xov->date;
+   h->from = xov->from;
+   h->date = xov->date_malloced;
    h->refs = xov->references;
    h->lines = xov->lines;
    h->bytes = xov->bytes;
@@ -893,6 +901,7 @@ int slrn_read_xover (Slrn_XOver_Type *xov) /*{{{*/
    if (Slrn_Server_Obj->sv_has_xover && !Suspend_XOver_For_Kill &&
        (Slrn_Prefer_Head != 2))
      {
+	int bytes = -1;
 	while (1)
 	  {
 	     status = Slrn_Server_Obj->sv_read_line (buf, sizeof (buf));
@@ -905,6 +914,12 @@ int slrn_read_xover (Slrn_XOver_Type *xov) /*{{{*/
 	       }
 
 	     id = atoi (buf);
+# if SLRN_HAS_SPOOL_SUPPORT
+	     if (Slrn_Spool_Check_Up_On_Nov)
+	       if (-1 == (bytes = Slrn_Server_Obj->sv_get_article_size (id)))
+		 continue; /* skip nonexisting article */
+# endif
+	     
 	     if ((id >= XOver_Min) && (id <= XOver_Max))
 	       break;
 	     /* else server screwed up and gave bad response.  Ignore it. */
@@ -913,6 +928,8 @@ int slrn_read_xover (Slrn_XOver_Type *xov) /*{{{*/
 	if (-1 == parse_xover_line (buf, xov))
 	  return -1;
 	
+	if ((xov->bytes == 0) && (bytes > 0))
+	  xov->bytes = bytes;
 	return 1;
      }
 
@@ -1106,7 +1123,7 @@ int slrn_read_add_xover (Slrn_Header_Line_Type **l) /*{{{*/
    if (status == 1)
      {
 	slrn_free (xov.subject_malloced);
-	slrn_free (xov.from_malloced);
+	slrn_free (xov.date_malloced);
 	slrn_free_additional_headers (xov.add_hdrs);
 	return xov.id;
      }
