@@ -2,7 +2,7 @@
  This file is part of SLRN.
 
  Copyright (c) 1994, 1999 John E. Davis <davis@space.mit.edu>
- Copyright (c) 2001 Thomas Schultz <tststs@gmx.de>
+ Copyright (c) 2001-2003 Thomas Schultz <tststs@gmx.de>
 
  This program is free software; you can redistribute it and/or modify it
  under the terms of the GNU General Public License as published by the Free
@@ -29,9 +29,9 @@
 typedef struct
 {
    int id;
-   char *subject_malloced; /* keep this separate from the rest */
-   char *from_malloced;
-   char *date;
+   char *subject_malloced; /* keep these separate from the rest */
+   char *from;
+   char *date_malloced;
    char *message_id;
    char *references;
    char *xref;
